@@ -75,6 +75,7 @@ html_theme_options = {
         "panelitelink",
         "page-toc",
     ],
+    "announcement": "Panel 1.4 has just been released! Checkout the <a href='https://panel.holoviz.org/about/releases.html#version-1-4-0'>release notes</a> and support Panel by giving it a 🌟 on <a href='https://github.com/holoviz/panel'>Github</a>.",
 }
 
 extensions += [
@@ -89,9 +90,9 @@ napoleon_numpy_docstring = True
 myst_enable_extensions = ["colon_fence", "deflist"]
 
 gallery_endpoint = 'panel-gallery-dev' if is_dev else 'panel-gallery'
-gallery_url = f'https://{gallery_endpoint}.pyviz.demo.anaconda.com'
-jlite_url = 'https://pyviz-dev.github.io/panelite-dev' if is_dev else 'https://panelite.holoviz.org'
-pyodide_url = 'https://pyviz-dev.github.io/panel/pyodide' if is_dev else 'https://panel.holoviz.org/pyodide'
+gallery_url = f'https://{gallery_endpoint}.holoviz.dsp.anaconda.com'
+jlite_url = 'https://holoviz-dev.github.io/panelite-dev' if is_dev else 'https://panelite.holoviz.org'
+pyodide_url = 'https://holoviz-dev.github.io/panel/pyodide' if is_dev else 'https://panel.holoviz.org/pyodide'
 
 nbsite_analytics = {
     'goatcounter_holoviz': True,
@@ -176,6 +177,10 @@ nbbuild_patterns_to_take_along = ["simple.html", "*.json", "json_*"]
 # Override the Sphinx default title that appends `documentation`
 html_title = f'{project} v{version}'
 
+# Ensure the global version string includes the SHA to ensure the service
+# worker is invalidated on builds between tags
+if is_dev:
+    version = panel.__version__
 
 # Patching GridItemCardDirective to be able to substitute the domain name
 # in the link option.
