@@ -23,6 +23,7 @@ from ..widgets.base import Widget
 from ..widgets.button import Button
 from ..widgets.input import FileInput, TextInput
 from .feed import CallbackState, ChatFeed
+from .input import ChatAreaInput
 from .message import ChatMessage, _FileInputMessage
 
 
@@ -147,7 +148,7 @@ class ChatInterface(ChatFeed):
     def __init__(self, *objects, **params):
         widgets = params.get("widgets")
         if widgets is None:
-            params["widgets"] = [TextInput(placeholder="Send a message")]
+            params["widgets"] = [ChatAreaInput(placeholder="Send a message")]
         elif not isinstance(widgets, list):
             params["widgets"] = [widgets]
         active = params.pop("active", None)
@@ -267,7 +268,7 @@ class ChatInterface(ChatFeed):
             # TextAreaInput will trigger auto send!
             auto_send = (
                 isinstance(widget, tuple(self.auto_send_types)) or
-                type(widget) is TextInput
+                type(widget) in (TextInput, ChatAreaInput)
             )
             if auto_send and widget in new_widgets:
                 callback = partial(self._button_data["send"].callback, self)
@@ -276,6 +277,8 @@ class ChatInterface(ChatFeed):
                 sizing_mode="stretch_width",
                 css_classes=["chat-interface-input-widget"]
             )
+            if isinstance(widget, ChatAreaInput):
+                self.link(widget, disabled="disabled_enter")
 
             self._buttons = {}
             for button_data in self._button_data.values():
@@ -366,7 +369,12 @@ class ChatInterface(ChatFeed):
             return
 
         active_widget = self.active_widget
+        # value_input for ChatAreaInput because value is unsynced until "Enter",
+        # value for TextInput and others
         value = active_widget.value
+        if not value and hasattr(active_widget, "value_input"):
+            value = active_widget.value_input
+
         if value:
             if isinstance(active_widget, FileInput):
                 value = _FileInputMessage(
@@ -380,7 +388,8 @@ class ChatInterface(ChatFeed):
                 if hasattr(active_widget, "value_input"):
                     updates["value_input"] = ""
                 try:
-                    active_widget.param.update(updates)
+                    with param.discard_events(self):
+                        active_widget.param.update(updates)
                 except ValueError:
                     pass
         else:
@@ -582,7 +591,7 @@ class ChatInterface(ChatFeed):
     @param.depends("_callback_state", watch=True)
     async def _update_input_disabled(self):
         busy_states = (CallbackState.RUNNING, CallbackState.GENERATING)
-        if not self.show_stop or self._callback_state not in busy_states:
+        if not self.show_stop or self._callback_state not in busy_states or self._callback_future is None:
             with param.parameterized.batch_call_watchers(self):
                 self._buttons["send"].visible = True
                 self._buttons["stop"].visible = False
@@ -597,3 +606,83 @@ class ChatInterface(ChatFeed):
         """
         await super()._cleanup_response()
         await self._update_input_disabled()
+
+
+    def send(
+        self,
+        value: ChatMessage | dict | Any,
+        user: str | None = None,
+        avatar: str | bytes | BytesIO | None = None,
+        respond: bool = True,
+    ) -> ChatMessage | None:
+        """
+        Sends a value and creates a new message in the chat log.
+
+        If `respond` is `True`, additionally executes the callback, if provided.
+
+        Arguments
+        ---------
+        value : ChatMessage | dict | Any
+            The message contents to send.
+        user : str | None
+            The user to send as; overrides the message message's user if provided.
+            Will default to the user parameter.
+        avatar : str | bytes | BytesIO | None
+            The avatar to use; overrides the message message's avatar if provided.
+            Will default to the avatar parameter.
+        respond : bool
+            Whether to execute the callback.
+
+        Returns
+        -------
+        The message that was created.
+        """
+        if not isinstance(value, ChatMessage):
+            if user is None:
+                user = self.user
+            if avatar is None:
+                avatar = self.avatar
+        return super().send(value, user=user, avatar=avatar, respond=respond)
+
+    def stream(
+        self,
+        value: str | dict | ChatMessage,
+        user: str | None = None,
+        avatar: str | bytes | BytesIO | None = None,
+        message: ChatMessage | None = None,
+        replace: bool = False,
+    ) -> ChatMessage | None:
+        """
+        Streams a token and updates the provided message, if provided.
+        Otherwise creates a new message in the chat log, so be sure the
+        returned message is passed back into the method, e.g.
+        `message = chat.stream(token, message=message)`.
+
+        This method is primarily for outputs that are not generators--
+        notably LangChain. For most cases, use the send method instead.
+
+        Arguments
+        ---------
+        value : str | dict | ChatMessage
+            The new token value to stream.
+        user : str | None
+            The user to stream as; overrides the message's user if provided.
+            Will default to the user parameter.
+        avatar : str | bytes | BytesIO | None
+            The avatar to use; overrides the message's avatar if provided.
+            Will default to the avatar parameter.
+        message : ChatMessage | None
+            The message to update.
+        replace : bool
+            Whether to replace the existing text when streaming a string or dict.
+
+        Returns
+        -------
+        The message that was updated.
+        """
+        if not isinstance(value, ChatMessage):
+            # ChatMessage cannot set user or avatar when explicitly streaming
+            # so only set to the default when not a ChatMessage
+            user = user or self.user
+            avatar = avatar or self.avatar
+        return super().stream(value, user=user, avatar=avatar, message=message, replace=replace)
