@@ -1,12 +1,13 @@
 from __future__ import annotations
 
 import asyncio
+import html
 import os
 import weakref
 
 from dataclasses import dataclass
 from typing import (
-    Any, Dict, List, Union,
+    TYPE_CHECKING, Any, Dict, List, Union,
 )
 
 import tornado
@@ -25,11 +26,14 @@ from bokeh.util.token import get_session_id, get_token_payload
 from ipykernel.comm import Comm
 
 from ..util import edit_readonly
-from .markdown import build_single_handler_application
+from .application import build_single_handler_application
 from .resources import Resources
 from .server import server_html_page_for_session
 from .state import set_curdoc, state
 
+if TYPE_CHECKING:
+    from bokeh.document.events import DocumentPatchedEvent
+
 
 @dataclass
 class _RequestProxy:
@@ -47,6 +51,20 @@ class Mimebundle:
         return self._mimebundle, {}
 
 
+class JupyterServerSession(ServerSession):
+
+    _tasks = set()
+
+    def _document_patched(self, event: DocumentPatchedEvent) -> None:
+        may_suppress = event.setter is self
+        for connection in self._subscribed_connections:
+            if may_suppress and connection is self._current_patch_connection:
+                continue
+            task = asyncio.ensure_future(connection.send_patch_document(event))
+            self._tasks.add(task)
+            task.add_done_callback(self._tasks.discard)
+
+
 class PanelExecutor(WSHandler):
     """
     The PanelExecutor is intended to be run inside a kernel where it
@@ -55,6 +73,8 @@ class PanelExecutor(WSHandler):
     to send and receive messages to and from the frontend.
     """
 
+    _tasks = set()
+
     def __init__(self, path, token, root_url):
         self.path = path
         self.token = token
@@ -75,7 +95,7 @@ class PanelExecutor(WSHandler):
         )
         self._set_state()
         try:
-            self.session = self._create_server_session()
+            self.session, self._error = self._create_server_session()
             self.connection = ServerConnection(self.protocol, self, None, self.session)
         except Exception as e:
             self.exception = e
@@ -92,13 +112,14 @@ class PanelExecutor(WSHandler):
         return payload
 
     def _set_state(self):
-        state._jupyter_kernel_context = True
         with edit_readonly(state):
             state.base_url = self.root_url + '/'
             state.rel_path = self.root_url
 
     def _receive_msg(self, msg) -> None:
-        asyncio.ensure_future(self._receive_msg_async(msg))
+        task = asyncio.ensure_future(self._receive_msg_async(msg))
+        self._tasks.add(task)
+        task.add_done_callback(self._tasks.discard)
 
     async def _receive_msg_async(self, msg) -> None:
         try:
@@ -141,6 +162,7 @@ class PanelExecutor(WSHandler):
         # expose the session context to the document
         # use the _attribute to set the public property .session_context
         doc._session_context = weakref.ref(session_context)
+        state._jupyter_kernel_context = session_context
 
         if self.path.endswith('.yaml') or self.path.endswith('.yml'):
             from lumen.command import (
@@ -152,20 +174,28 @@ class PanelExecutor(WSHandler):
         with set_curdoc(doc):
             app.initialize_document(doc)
 
+        runner = app._handlers[0]._runner
         loop = tornado.ioloop.IOLoop.current()
-        session = ServerSession(self.session_id, doc, io_loop=loop, token=self.token)
+        session = JupyterServerSession(self.session_id, doc, io_loop=loop, token=self.token)
         session_context._set_session(session)
-        return session
+        return session, runner.error_detail
 
     async def write_message(
         self, message: Union[bytes, str, Dict[str, Any]],
         binary: bool = False, locked: bool = True
     ) -> None:
         metadata = {'binary': binary}
-        if binary:
-            self.comm.send({}, metadata=metadata, buffers=[message])
+        if locked:
+            with await self.write_lock.acquire():
+                if binary:
+                    self.comm.send({}, metadata=metadata, buffers=[message])
+                else:
+                    self.comm.send(message, metadata=metadata)
         else:
-            self.comm.send(message, metadata=metadata)
+            if binary:
+                self.comm.send({}, metadata=metadata, buffers=[message])
+            else:
+                self.comm.send(message, metadata=metadata)
 
     def render(self) -> Mimebundle:
         """
@@ -176,6 +206,18 @@ class PanelExecutor(WSHandler):
             return Mimebundle({'text/error': f'Session did not start correctly: {self.exception}'})
         with set_curdoc(self.session.document):
             if not self.session.document.roots:
+                if self._error:
+                    lines = self._error.split('\n')
+                    error = html.escape('\n'.join(lines[:1]+lines[3:]))
+                    return Mimebundle({
+                        'text/error': (
+                            "<span>Application errored while starting up:</span>"
+                            "<br><br>"
+                            "<div style='overflow: auto'>"
+                            f"  <code style='font-size: 0.75em; white-space: pre;'>{error}</code>"
+                            "</div>"
+                        )
+                    })
                 return Mimebundle({
                     'text/error': (
                         "The Panel application being served did not serve any contents. "
@@ -183,7 +225,7 @@ class PanelExecutor(WSHandler):
                         "script with .servable(), e.g. pn.Row('Hello World!').servable()."
                     )
                 })
-            html = server_html_page_for_session(
+            html_page = server_html_page_for_session(
                 self.session,
                 resources=self.resources,
                 title=self.session.document.title,
@@ -191,7 +233,7 @@ class PanelExecutor(WSHandler):
                 template_variables=self.session.document.template_variables
             )
         return Mimebundle({
-            'text/html': html,
+            'text/html': html_page,
             'application/bokeh-extensions': {
                 name: str(ext) for name, ext in extension_dirs.items()
             }
