@@ -56,6 +56,7 @@ from tornado.web import StaticFileHandler
 
 from ..config import config
 from .resources import DIST_DIR, ERROR_TEMPLATE, _env
+from .server import COMPONENT_PATH, ComponentResourceHandler
 from .state import state
 
 logger = logging.getLogger(__name__)
@@ -111,6 +112,7 @@ import os
 import pathlib
 import sys
 
+os.environ['BOKEH_RESOURCES'] = 'server'
 app = r'{{ path }}'
 os.chdir(str(pathlib.Path(app).parent))
 sys.path = [os.getcwd()] + sys.path[1:]
@@ -144,7 +146,37 @@ def generate_executor(path: str, token: str, root_url: str) -> str:
         execute_template.render(path=path, token=token, root_url=root_url)
     )
 
-class PanelJupyterHandler(JupyterHandler):
+class PanelBaseHandler(JupyterHandler):
+
+    def initialize(self, **kwargs):
+        super().initialize(**kwargs)
+        self.notebook_path = kwargs.pop('notebook_path', [])
+
+    def nb_path(self, path=None):
+        root_dir = get_server_root_dir(self.application.settings)
+        rel_path = pathlib.Path(self.notebook_path or path)
+        if rel_path.is_absolute():
+            notebook_path = str(rel_path)
+        else:
+            notebook_path = str((root_dir / rel_path).absolute())
+        return pathlib.Path(notebook_path)
+
+
+class PanelLayoutHandler(PanelBaseHandler):
+
+    @tornado.web.authenticated
+    async def get(self, path=None):
+        nb_path = self.nb_path(path)
+        layout_path = nb_path.parent / f'.{nb_path.name}.layout'
+        if not layout_path.is_file():
+            return {}
+        with open(layout_path, encoding='utf-8') as f:
+            layout = json.load(f)
+        self.set_header('Content-Type', 'text/json')
+        self.finish(layout)
+
+
+class PanelJupyterHandler(PanelBaseHandler):
     """
     The PanelJupyterHandler expects to be given a path to a notebook,
     .py file or Lumen .yaml file. Based on the kernelspec in the
@@ -158,7 +190,6 @@ class PanelJupyterHandler(JupyterHandler):
 
     def initialize(self, **kwargs):
         super().initialize(**kwargs)
-        self.notebook_path = kwargs.pop('notebook_path', [])
         self.kernel_started = False
 
     async def _get_info(self, msg_id, timeout=KERNEL_TIMEOUT):
@@ -192,13 +223,7 @@ class PanelJupyterHandler(JupyterHandler):
 
     @tornado.web.authenticated
     async def get(self, path=None):
-        root_dir = get_server_root_dir(self.application.settings)
-        rel_path = pathlib.Path(self.notebook_path or path)
-        if rel_path.is_absolute():
-            notebook_path = str(rel_path)
-        else:
-            notebook_path = str((root_dir / rel_path).absolute())
-
+        notebook_path = self.nb_path(path)
         if (
             self.notebook_path and path
         ):  # when we are in single notebook mode but have a path
@@ -217,7 +242,7 @@ class PanelJupyterHandler(JupyterHandler):
         # Provision a kernel with the desired kernelspec
         if self.request.arguments.get('kernel'):
             requested_kernel = self.request.arguments.pop('kernel')[0].decode('utf-8')
-        elif notebook_path.endswith('.ipynb'):
+        elif notebook_path.suffix == '.ipynb':
             with open(notebook_path) as f:
                 nb = json.load(f)
             requested_kernel = nb.get('metadata', {}).get('kernelspec', {}).get('name')
@@ -231,7 +256,7 @@ class PanelJupyterHandler(JupyterHandler):
                 html = KERNEL_ERROR_TEMPLATE.render(
                     base_url=f'{root_url}/',
                     kernels=available_kernels,
-                    error_type='Kernel error',
+                    error_type='Kernel Error',
                     error=f"No such kernel '{requested_kernel}'",
                     title='Panel: Kernel not found'
                 )
@@ -277,8 +302,8 @@ class PanelJupyterHandler(JupyterHandler):
             html = ERROR_TEMPLATE.render(
                 npm_cdn=config.npm_cdn,
                 base_url=f'{root_url}/',
-                error_type="Kernel error",
-                error="Failed to start kernel",
+                error_type="Kernel Error",
+                error="Failed to start application",
                 error_msg=str(e),
                 title="Panel: Kernel Error"
             )
@@ -306,6 +331,8 @@ class PanelWSProxy(WSHandler, JupyterHandler):
     receives Bokeh protocol messages via a Jupyter Comm.
     """
 
+    _tasks = set()
+
     def __init__(self, tornado_app, *args, **kw) -> None:
         # Note: tornado_app is stored as self.application
         kw['application_context'] = None
@@ -373,7 +400,9 @@ class PanelWSProxy(WSHandler, JupyterHandler):
         await self.send_message(msg)
 
         self._ping_job.start()
-        asyncio.create_task(self._check_for_message())
+        task = asyncio.create_task(self._check_for_message())
+        self._tasks.add(task)
+        task.add_done_callback(self._tasks.discard)
 
     async def _check_for_message(self):
         while True:
@@ -445,8 +474,12 @@ def _load_jupyter_server_extension(notebook_app):
              PanelWSProxy),
             (urljoin(base_url, r"panel-preview/render/(.*)"),
              PanelJupyterHandler, {}),
+            (urljoin(base_url, r"panel-preview/layout/(.*)"),
+             PanelLayoutHandler, {}),
             (urljoin(base_url, r"panel_dist/(.*)"),
-             StaticFileHandler, dict(path=DIST_DIR))
+             StaticFileHandler, dict(path=DIST_DIR)),
+            (urljoin(base_url, f'panel-preview/{COMPONENT_PATH}(.*)'),
+             ComponentResourceHandler, {})
         ]
     )
 
