@@ -32,6 +32,9 @@ from bokeh.models import Model
 from bokeh.resources import CDN, INLINE
 from bokeh.settings import _Unset, settings
 from bokeh.util.serialization import make_id
+from param.display import (
+    register_display_accessor, unregister_display_accessor,
+)
 from pyviz_comms import (
     PYVIZ_PROXY, Comm, JupyterCommJS,
     JupyterCommManager as _JupyterCommManager, nb_mime_js,
@@ -54,6 +57,7 @@ if TYPE_CHECKING:
     from ..widgets.base import Widget
     from .location import Location
 
+
 #---------------------------------------------------------------------
 # Private API
 #---------------------------------------------------------------------
@@ -301,6 +305,9 @@ def require_components():
 
     skip_import = {}
     for model in js_requires:
+        if not isinstance(model, dict) and issubclass(model, ReactiveHTML) and not model._loaded():
+            continue
+
         if hasattr(model, '__js_skip__'):
             skip_import.update(model.__js_skip__)
 
@@ -575,3 +582,14 @@ def ipywidget(obj: Any, doc=None, **kwargs: Any):
                 current[:] = [new_model]
         widget.observe(view_count_changed, '_view_count')
     return widget
+
+
+try:
+    unregister_display_accessor('_ipython_display_')
+except KeyError:
+    pass
+
+try:
+    register_display_accessor('_repr_mimebundle_', mime_renderer)
+except Exception:
+    pass
