@@ -14,7 +14,6 @@ import pathlib
 import re
 import textwrap
 
-from base64 import b64encode
 from collections import OrderedDict
 from contextlib import contextmanager
 from functools import lru_cache
@@ -26,8 +25,8 @@ from typing import (
 import param
 
 from bokeh.embed.bundle import (
-    CSS_RESOURCES as BkCSS_RESOURCES, Bundle as BkBundle, _bundle_extensions,
-    _use_mathjax, bundle_models, extension_dirs,
+    CSS_RESOURCES as BkCSS_RESOURCES, URL, Bundle as BkBundle,
+    _bundle_extensions, _use_mathjax, bundle_models, extension_dirs,
 )
 from bokeh.model import Model
 from bokeh.models import ImportedStyleSheet
@@ -39,7 +38,6 @@ from markupsafe import Markup
 
 from ..config import config, panel_extension as extension
 from ..util import isurl, url_path
-from .loading import LOADING_INDICATOR_CSS_CLASS
 from .state import state
 
 if TYPE_CHECKING:
@@ -71,10 +69,16 @@ def get_env():
 def conffilter(value):
     return json.dumps(OrderedDict(value)).replace('"', '\'')
 
+class json_dumps(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, URL):
+            return str(obj)
+        return super().default(obj)
+
 _env = get_env()
 _env.trim_blocks = True
 _env.lstrip_blocks = True
-_env.filters['json'] = lambda obj: Markup(json.dumps(obj))
+_env.filters['json'] = lambda obj: Markup(json.dumps(obj, cls=json_dumps))
 _env.filters['conffilter'] = conffilter
 _env.filters['sorted'] = sorted
 
@@ -182,13 +186,11 @@ def process_raw_css(raw_css):
 
 @lru_cache(maxsize=None)
 def loading_css(loading_spinner, color, max_height):
-    with open(ASSETS_DIR / f'{loading_spinner}_spinner.svg', encoding='utf-8') as f:
-        svg = f.read().replace('\n', '').format(color=color)
-    b64 = b64encode(svg.encode('utf-8')).decode('utf-8')
     return textwrap.dedent(f"""
-    :host(.{LOADING_INDICATOR_CSS_CLASS}.pn-{loading_spinner}):before, .pn-loading.pn-{loading_spinner}:before {{
-      background-image: url("data:image/svg+xml;base64,{b64}");
-      background-size: auto calc(min(50%, {max_height}px));
+    :host(.pn-loading):before, .pn-loading:before {{
+      background-color: {color};
+      mask-size: auto calc(min(50%, {max_height}px));
+      -webkit-mask-size: auto calc(min(50%, {max_height}px));
     }}""")
 
 def resolve_custom_path(
@@ -253,16 +255,24 @@ def component_resource_path(component, attr, path):
 def patch_stylesheet(stylesheet, dist_url):
     url = stylesheet.url
     if url.startswith(CDN_DIST+dist_url) and dist_url != CDN_DIST:
-        patched_url = url.replace(CDN_DIST+dist_url, dist_url) + f'?v={JS_VERSION}'
+        patched_url = url.replace(CDN_DIST+dist_url, dist_url)
     elif url.startswith(CDN_DIST) and dist_url != CDN_DIST:
-        patched_url = url.replace(CDN_DIST, dist_url) + f'?v={JS_VERSION}'
+        patched_url = url.replace(CDN_DIST, dist_url)
+    elif url.startswith(LOCAL_DIST) and dist_url.lstrip('./').startswith(LOCAL_DIST):
+        patched_url = url.replace(LOCAL_DIST, dist_url)
     else:
         return
+    version_suffix = f'?v={JS_VERSION}'
+    if not patched_url.endswith(version_suffix):
+        patched_url += version_suffix
     try:
         stylesheet.url = patched_url
     except Exception:
         pass
 
+def _is_file_path(stylesheet: str)->bool:
+    return stylesheet.lower().endswith(".css")
+
 def resolve_stylesheet(cls, stylesheet: str, attribute: str | None = None):
     """
     Resolves a stylesheet definition, e.g. originating on a component
@@ -271,6 +281,7 @@ def resolve_stylesheet(cls, stylesheet: str, attribute: str | None = None):
 
     - Absolute URL defined with http(s) protocol
     - A path relative to the component
+    - A raw css string
 
     Arguments
     ---------
@@ -280,7 +291,7 @@ def resolve_stylesheet(cls, stylesheet: str, attribute: str | None = None):
         The stylesheet definition
     """
     stylesheet = str(stylesheet)
-    if not stylesheet.startswith('http') and attribute and (custom_path:= resolve_custom_path(cls, stylesheet)):
+    if not stylesheet.startswith('http') and attribute and _is_file_path(stylesheet) and (custom_path:= resolve_custom_path(cls, stylesheet)):
         if not state._is_pyodide and state.curdoc and state.curdoc.session_context:
             stylesheet = component_resource_path(cls, attribute, stylesheet)
         else:
@@ -366,7 +377,7 @@ def bundle_resources(roots, resources, notebook=False, reloading=False, enable_m
     elif roots:
         use_mathjax = _use_mathjax(roots) or 'mathjax' in ext._loaded_extensions
     else:
-        use_mathjax = False
+        use_mathjax = 'mathjax' in ext._loaded_extensions
 
     if js_resources:
         js_resources = js_resources.clone()
@@ -384,7 +395,7 @@ def bundle_resources(roots, resources, notebook=False, reloading=False, enable_m
     extensions = _bundle_extensions(None, js_resources)
     if reloading:
         extensions = [
-            ext for ext in extensions if not (ext.cdn_url is not None and ext.cdn_url.startswith('https://unpkg.com/@holoviz/panel@'))
+            ext for ext in extensions if not (ext.cdn_url is not None and str(ext.cdn_url).startswith('https://unpkg.com/@holoviz/panel@'))
         ]
 
     extra_js = []
@@ -393,6 +404,8 @@ def bundle_resources(roots, resources, notebook=False, reloading=False, enable_m
     elif mode == "server":
         for bundle in extensions:
             server_url = bundle.server_url
+            if not isinstance(server_url, str):
+                server_url = str(server_url)
             if resources.root_url and not resources.absolute:
                 server_url = server_url.replace(resources.root_url, '', 1)
                 if state.rel_path:
@@ -413,6 +426,10 @@ def bundle_resources(roots, resources, notebook=False, reloading=False, enable_m
         js_raw.append(ext)
 
     hashes = js_resources.hashes if js_resources else {}
+
+    js_files = list(map(URL, js_files))
+    css_files = list(map(URL, css_files))
+
     return Bundle(
         css_files=css_files,
         css_raw=css_raw,
@@ -471,7 +488,11 @@ class ResourceComponent:
                 cls, f'_resources/{resource_type}', resource
             )
 
-    def resolve_resources(self, cdn: bool | Literal['auto'] = 'auto') -> ResourcesType:
+    def resolve_resources(
+        self,
+        cdn: bool | Literal['auto'] = 'auto',
+        extras: dict[str, dict[str, str]] | None = None
+    ) -> ResourcesType:
         """
         Resolves the resources required for this component.
 
@@ -481,6 +502,9 @@ class ResourceComponent:
             Whether to load resources from CDN or local server. If set
             to 'auto' value will be automatically determine based on
             global settings.
+        extras: dict[str, dict[str, str]] | None
+            Additional resources to add to the bundle. Valid resource
+            types include js, js_modules and css.
 
         Returns
         -------
@@ -502,7 +526,6 @@ class ResourceComponent:
             else:
                 resources[rt] = res
 
-        cdn = use_cdn() if cdn == 'auto' else cdn
         resource_types: ResourcesType = {
             'js': {},
             'js_modules': {},
@@ -510,6 +533,7 @@ class ResourceComponent:
             'raw_css': []
         }
 
+        cdn = use_cdn() if cdn == 'auto' else cdn
         for resource_type in resource_types:
             if resource_type not in resources or resource_type == 'raw_css':
                 continue
@@ -520,6 +544,18 @@ class ResourceComponent:
                 )
                 if resolved_resource:
                     resource_files[rname] = resolved_resource
+
+        version_suffix = f'?v={JS_VERSION}'
+        dist_path = get_dist_path(cdn=cdn)
+        for resource_type, extra_resources in (extras or {}).items():
+            resource_files = resource_types[resource_type]
+            for name, res in extra_resources.items():
+                if not cdn:
+                    res = res.replace(CDN_DIST, dist_path)
+                    if not res.endswith(version_suffix):
+                        res += version_suffix
+                resource_files[name] = res
+
         return resource_types
 
 
@@ -588,7 +624,7 @@ class Resources(BkResources):
             if not (getattr(model, resource_type, None) and model._loaded()):
                 continue
             for resource in getattr(model, resource_type, []):
-                if not isurl(resource) and not resource.startswith('static/extensions'):
+                if not isurl(resource) and not resource.lstrip('./').startswith('static/extensions'):
                     resource = component_resource_path(model, resource_type, resource)
                 if resource not in resources:
                     resources.append(resource)
@@ -598,8 +634,11 @@ class Resources(BkResources):
         Computes relative and absolute paths for resources.
         """
         new_resources = []
+        version_suffix = f'?v={JS_VERSION}'
         cdn_base = f'{config.npm_cdn}/@holoviz/panel@{JS_VERSION}/dist/'
         for resource in resources:
+            if not isinstance(resource, str):
+                resource = str(resource)
             resource = resource.replace('https://unpkg.com', config.npm_cdn)
             if resource.startswith(cdn_base):
                 resource = resource.replace(cdn_base, CDN_DIST)
@@ -612,6 +651,8 @@ class Resources(BkResources):
                     resource = f'{state.rel_path}/{resource}'
                 elif self.absolute and self.mode == 'server':
                     resource = f'{self.root_url}{resource}'
+            if resource.endswith('.css'):
+                resource += version_suffix
             new_resources.append(resource)
         return new_resources
 
@@ -674,7 +715,7 @@ class Resources(BkResources):
         # Inline local dist resources
         css_files = self._collect_external_resources("__css__")
         self.extra_resources(css_files, '__css__')
-        if self.mode.lower() != 'cdn':
+        if self.mode.lower() not in ('server', 'cdn'):
             raw += [
                 (DIST_DIR / css.replace(CDN_DIST, '')).read_text(encoding='utf-8')
                 for css in css_files if is_cdn_url(css)
@@ -733,6 +774,10 @@ class Resources(BkResources):
         from ..reactive import ReactiveHTML
 
         modules = list(config.js_modules.values())
+        for model in Model.model_class_reverse_map.values():
+            if hasattr(model, '__javascript_modules__'):
+                modules.extend(model.__javascript_modules__)
+
         self.extra_resources(modules, '__javascript_modules__')
         if config.design:
             design_resources = config.design().resolve_resources(
@@ -760,7 +805,7 @@ class Resources(BkResources):
         for model in Model.model_class_reverse_map.values():
             if hasattr(model, '__javascript_module_exports__'):
                 modules.update(dict(zip(model.__javascript_module_exports__, model.__javascript_modules__)))
-        return modules
+        return dict(zip(modules, self.adjust_paths(modules.values())))
 
     @property
     def js_raw(self):
