@@ -5,8 +5,6 @@ from __future__ import annotations
 
 import asyncio
 import datetime as dt
-import gc
-import html
 import importlib
 import inspect
 import logging
@@ -15,9 +13,7 @@ import pathlib
 import signal
 import sys
 import threading
-import traceback
 import uuid
-import weakref
 
 from collections import OrderedDict
 from contextlib import contextmanager
@@ -30,15 +26,11 @@ from typing import (
 from urllib.parse import urljoin, urlparse
 
 import bokeh
-import bokeh.command.util
 import param
 import tornado
 
 # Bokeh imports
 from bokeh.application import Application as BkApplication
-from bokeh.application.handlers.code import (
-    CodeHandler, _monkeypatch_io, patch_curdoc,
-)
 from bokeh.application.handlers.function import FunctionHandler
 from bokeh.core.json_encoder import serialize_json
 from bokeh.core.templates import AUTOLOAD_JS, FILE, MACROS
@@ -48,7 +40,6 @@ from bokeh.embed.bundle import Script
 from bokeh.embed.elements import script_for_render_items
 from bokeh.embed.util import RenderItem
 from bokeh.embed.wrappers import wrap_in_script_tag
-from bokeh.io import curdoc
 from bokeh.models import CustomJS
 from bokeh.server.server import Server as BokehServer
 from bokeh.server.urls import per_app_patterns, toplevel_patterns
@@ -73,15 +64,16 @@ from tornado.wsgi import WSGIContainer
 from ..config import config
 from ..util import edit_readonly, fullpath
 from ..util.warnings import warn
-from .document import init_doc, unlocked, with_lock  # noqa
+from .application import Application, build_single_handler_application
+from .document import (  # noqa
+    _cleanup_doc, init_doc, unlocked, with_lock,
+)
 from .liveness import LivenessHandler
 from .loading import LOADING_INDICATOR_CSS_CLASS
 from .logging import (
     LOG_SESSION_CREATED, LOG_SESSION_DESTROYED, LOG_SESSION_LAUNCHING,
 )
-from .markdown import build_single_handler_application
-from .profile import profile_ctx
-from .reload import autoreload_watcher
+from .reload import record_modules
 from .resources import (
     BASE_TEMPLATE, CDN_DIST, COMPONENT_PATH, ERROR_TEMPLATE, LOCAL_DIST,
     Resources, _env, bundle_resources, patch_model_css, resolve_custom_path,
@@ -217,7 +209,6 @@ state._on_session_created_internal.append(_initialize_session_info)
 # Bokeh patches
 #---------------------------------------------------------------------
 
-
 def html_page_for_render_items(
     bundle: Bundle | tuple[str, str], docs_json: dict[ID, DocJson],
     render_items: list[RenderItem], title: str, template: Template | str | None = None,
@@ -335,49 +326,13 @@ def autoload_js_script(doc, resources, token, element_id, app_path, absolute_url
 
     return AUTOLOAD_JS.render(bundle=bundle, elementid=element_id)
 
-def destroy_document(self, session):
-    """
-    Override for Document.destroy() without calling gc.collect directly.
-    The gc.collect() call is scheduled as a task, ensuring that when
-    multiple documents are destroyed in quick succession we do not
-    schedule excessive garbage collection.
-    """
-    if session is not None:
-        self.remove_on_change(session)
-
-    del self._roots
-    del self._theme
-    del self._template
-    self._session_context = None
-
-    self.callbacks.destroy()
-    self.models.destroy()
-    self.modules.destroy()
-
-    # Clear periodic callbacks
-    for cb in state._periodic.get(self, []):
-        cb.stop()
-
-    # Clean up pn.state to avoid tasks getting executed on dead session
-    for attr in dir(state):
-        # _param_watchers is deprecated in Param 2.0 and will raise a warning
-        if not attr.startswith('_') or attr == "_param_watchers":
-            continue
-        state_obj = getattr(state, attr)
-        if isinstance(state_obj, weakref.WeakKeyDictionary) and self in state_obj:
-            del state_obj[self]
-
-    # Schedule GC
-    at = dt.datetime.now() + dt.timedelta(seconds=5)
-    state.schedule_task('gc.collect', gc.collect, at=at)
-
-    del self.destroy
 
 # Patch Server to attach task factory to asyncio loop and handle Admin server context
 class Server(BokehServer):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self._autoreload_stop_event = None
         if state._admin_context:
             state._admin_context._loop = self._loop
 
@@ -385,39 +340,25 @@ class Server(BokehServer):
         super().start()
         if state._admin_context:
             self._loop.add_callback(state._admin_context.run_load_hook)
+        if config.autoreload:
+            from .reload import setup_autoreload_watcher
+            self._autoreload_stop_event = stop_event = asyncio.Event()
+            self._autoreload_task = self._loop.asyncio_loop.create_task(setup_autoreload_watcher(stop_event))
 
     def stop(self, wait: bool = True) -> None:
+        if self._autoreload_stop_event:
+            # For the stop event to be processed we have to restart
+            # the IOLoop briefly, ensuring an orderly cleanup
+            async def stop_autoreload():
+                self._autoreload_stop_event.set()
+                await self._autoreload_task
+            self._loop.asyncio_loop.run_until_complete(stop_autoreload())
         super().stop(wait=wait)
         if state._admin_context:
             state._admin_context.run_unload_hook()
 
 bokeh.server.server.Server = Server
 
-
-# Patch Application to handle session callbacks
-class Application(BkApplication):
-
-    def __init__(self, *args, **kwargs):
-        self._admin = kwargs.pop('admin', None)
-        super().__init__(*args, **kwargs)
-
-    async def on_session_created(self, session_context):
-        with set_curdoc(session_context._document):
-            if self._admin is not None:
-                config._admin = self._admin
-            for cb in state._on_session_created_internal+state._on_session_created:
-                cb(session_context)
-        await super().on_session_created(session_context)
-
-    def initialize_document(self, doc):
-        super().initialize_document(doc)
-        if doc in state._templates and doc not in state._templates[doc]._documents:
-            template = state._templates[doc]
-            with set_curdoc(doc):
-                template.server_doc(title=template.title, location=True, doc=doc)
-
-bokeh.command.util.Application = Application # type: ignore
-
 class SessionPrefixHandler:
 
     @contextmanager
@@ -462,7 +403,6 @@ class LoginUrlMixin:
         raise RuntimeError('login_url or get_login_url() must be supplied when authentication hooks are enabled')
 
 
-# Patch Bokeh DocHandler URL
 class DocHandler(LoginUrlMixin, BkDocHandler, SessionPrefixHandler):
 
     @authenticated
@@ -673,6 +613,39 @@ class RootHandler(LoginUrlMixin, BkRootHandler):
 toplevel_patterns[0] = (r'/?', RootHandler)
 bokeh.server.tornado.RootHandler = RootHandler
 
+# Copied from bokeh 2.4.0, to fix directly in bokeh at some point.
+def create_static_handler(prefix, key, app):
+    # patch
+    key = '/__patchedroot' if key == '/' else key
+
+    route = prefix
+    route += "/static/(.*)" if key == "/" else key + "/static/(.*)"
+    if app.static_path is not None:
+        return (route, StaticFileHandler, {"path" : app.static_path})
+    return (route, StaticHandler, {})
+
+bokeh.server.tornado.create_static_handler = create_static_handler
+
+#---------------------------------------------------------------------
+# Async patches
+#---------------------------------------------------------------------
+
+# Bokeh 2.4.x patches the asyncio event loop policy but Tornado 6.1
+# support the WindowsProactorEventLoopPolicy so we restore it,
+# unless we detect we are running on jupyter_server.
+if (
+    sys.platform == 'win32' and
+    tornado.version_info >= (6, 1) and
+    type(asyncio.get_event_loop_policy()) is asyncio.WindowsSelectorEventLoopPolicy and
+    (('jupyter_server' not in sys.modules and
+      'jupyter_client' not in sys.modules) or
+     'pytest' in sys.modules)
+):
+    asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
+
+#---------------------------------------------------------------------
+# Public API
+#---------------------------------------------------------------------
 
 class ComponentResourceHandler(StaticFileHandler):
     """
@@ -766,146 +739,6 @@ class ComponentResourceHandler(StaticFileHandler):
         return absolute_path
 
 
-def modify_document(self, doc: 'Document'):
-    from bokeh.io.doc import set_curdoc as bk_set_curdoc
-
-    from ..config import config
-
-    logger.info(LOG_SESSION_LAUNCHING, id(doc))
-
-    doc.on_event('document_ready', partial(state._schedule_on_load, doc))
-
-    if config.autoreload:
-        path = self._runner.path
-        argv = self._runner._argv
-        handler = type(self)(filename=path, argv=argv)
-        self._runner = handler._runner
-
-    module = self._runner.new_module()
-
-    # If no module was returned it means the code runner has some permanent
-    # unfixable problem, e.g. the configured source code has a syntax error
-    if module is None:
-        return
-
-    # One reason modules are stored is to prevent the module
-    # from being gc'd before the document is. A symptom of a
-    # gc'd module is that its globals become None. Additionally
-    # stored modules are used to provide correct paths to
-    # custom models resolver.
-    sys.modules[module.__name__] = module
-    doc.modules._modules.append(module)
-
-    try:
-        old_doc = curdoc()
-    except RuntimeError:
-        old_doc = None
-    bk_set_curdoc(doc)
-
-    if config.autoreload:
-        set_curdoc(doc)
-        state.onload(autoreload_watcher)
-
-    sessions = []
-
-    try:
-        def post_check():
-            newdoc = curdoc()
-            # Do not let curdoc track modules when autoreload is enabled
-            # otherwise it will erroneously complain that there is
-            # a memory leak
-            if config.autoreload:
-                newdoc.modules._modules = []
-
-            # script is supposed to edit the doc not replace it
-            if newdoc is not doc:
-                raise RuntimeError("%s at '%s' replaced the output document" % (self._origin, self._runner.path))
-
-        def handle_exception(handler, e):
-            from bokeh.application.handlers.handler import handle_exception
-
-            from ..pane import Alert
-
-            # Clean up
-            del sys.modules[module.__name__]
-
-            if hasattr(doc, 'modules'):
-                doc.modules._modules.remove(module)
-            else:
-                doc._modules.remove(module)
-            bokeh.application.handlers.code_runner.handle_exception = handle_exception
-            tb = html.escape(traceback.format_exc()).replace('\033[1m', '<b>').replace('\033[0m', '</b>')
-
-            # Serve error
-            e_msg = str(e).replace('\033[1m', '<b>').replace('\033[0m', '</b>')
-            Alert(
-                f'<b>{type(e).__name__}</b>: {e_msg}\n<pre style="overflow-y: auto">{tb}</pre>',
-                alert_type='danger', margin=5, sizing_mode='stretch_width'
-            ).servable()
-
-        if config.autoreload:
-            bokeh.application.handlers.code_runner.handle_exception = handle_exception
-
-        state._launching.append(doc)
-        with _monkeypatch_io(self._loggers):
-            with patch_curdoc(doc):
-                with profile_ctx(config.profiler) as sessions:
-                    self._runner.run(module, post_check)
-
-        def _log_session_destroyed(session_context):
-            logger.info(LOG_SESSION_DESTROYED, id(doc))
-
-        doc.on_session_destroyed(_log_session_destroyed)
-        doc.destroy = partial(destroy_document, doc) # type: ignore
-    finally:
-        state._launching.remove(doc)
-        if config.profiler:
-            try:
-                path = doc.session_context.request.path
-                state._profiles[(path, config.profiler)] += sessions
-                state.param.trigger('_profiles')
-            except Exception:
-                pass
-        if old_doc is not None:
-            bk_set_curdoc(old_doc)
-
-CodeHandler.modify_document = modify_document # type: ignore
-
-# Copied from bokeh 2.4.0, to fix directly in bokeh at some point.
-def create_static_handler(prefix, key, app):
-    # patch
-    key = '/__patchedroot' if key == '/' else key
-
-    route = prefix
-    route += "/static/(.*)" if key == "/" else key + "/static/(.*)"
-    if app.static_path is not None:
-        return (route, StaticFileHandler, {"path" : app.static_path})
-    return (route, StaticHandler, {})
-
-bokeh.server.tornado.create_static_handler = create_static_handler
-
-#---------------------------------------------------------------------
-# Async patches
-#---------------------------------------------------------------------
-
-# Bokeh 2.4.x patches the asyncio event loop policy but Tornado 6.1
-# support the WindowsProactorEventLoopPolicy so we restore it,
-# unless we detect we are running on jupyter_server.
-if (
-    sys.platform == 'win32' and
-    sys.version_info[:3] >= (3, 8, 0) and
-    tornado.version_info >= (6, 1) and
-    type(asyncio.get_event_loop_policy()) is asyncio.WindowsSelectorEventLoopPolicy and
-    (('jupyter_server' not in sys.modules and
-      'jupyter_client' not in sys.modules) or
-     'pytest' in sys.modules)
-):
-    asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
-
-#---------------------------------------------------------------------
-# Public API
-#---------------------------------------------------------------------
-
 def serve(
     panels: TViewableFuncOrPath | Mapping[str, TViewableFuncOrPath],
     port: int = 0,
@@ -978,10 +811,15 @@ def serve(
     ))
     if threaded:
         kwargs['loop'] = loop = IOLoop(make_current=False) if loop is None else loop
+        # To ensure that we have correspondence between state._threads and state._servers
+        # we must provide a server_id here
+        if 'server_id' not in kwargs:
+            kwargs['server_id'] = uuid.uuid4().hex
+
         server = StoppableThread(
             target=get_server, io_loop=loop, args=(panels,), kwargs=kwargs
         )
-        server_id = kwargs.get('server_id', uuid.uuid4().hex)
+        server_id = kwargs['server_id']
         state._threads[server_id] = server
         server.start()
     else:
@@ -1061,6 +899,7 @@ def get_server(
     logout_template: Optional[str] = None,
     session_history: Optional[int] = None,
     liveness: bool | str = False,
+    warm: bool = False,
     **kwargs
 ) -> Server:
     """
@@ -1149,6 +988,9 @@ def get_server(
       Whether to add a liveness endpoint. If a string is provided
       then this will be used as the endpoint, otherwise the endpoint
       will be hosted at /liveness.
+    warm: bool (optional, default=False)
+      Whether to run the applications before serving them to ensure
+      all imports and caches are fully warmed up before serving the app.
     kwargs: dict
       Additional keyword arguments to pass to Server instance.
 
@@ -1203,13 +1045,26 @@ def get_server(
     else:
         if isinstance(panel, pathlib.Path):
             panel = str(panel) # enables serving apps from Paths
-        if (isinstance(panel, str) and panel.endswith(('.py', '.ipynb', '.md'))
+        if isinstance(panel, BkApplication):
+            panel = {'/': app}
+        elif (isinstance(panel, str) and panel.endswith(('.py', '.ipynb', '.md'))
             and os.path.isfile(panel)):
             apps = {'/': build_single_handler_application(panel)}
         else:
             handler = FunctionHandler(partial(_eval_panel, panel, server_id, title, location, admin))
             apps = {'/': Application(handler, admin=admin)}
 
+    if warm or config.autoreload:
+        for app in apps.values():
+            if config.autoreload:
+                with record_modules(list(apps.values())):
+                    session = generate_session(app)
+            else:
+                session = generate_session(app)
+            with set_curdoc(session.document):
+                state._on_load(None)
+            _cleanup_doc(session.document, destroy=True)
+
     if admin:
         if '/admin' in apps:
             raise ValueError(
