@@ -1,10 +1,11 @@
-import {Tooltip, TooltipView} from "@bokehjs/models/ui/tooltip"
-import { build_view, IterViews} from "@bokehjs/core/build_views"
-import * as p from "@bokehjs/core/properties"
+import type {TooltipView} from "@bokehjs/models/ui/tooltip"
+import {Tooltip} from "@bokehjs/models/ui/tooltip"
+import type {IterViews} from "@bokehjs/core/build_views"
+import {build_view} from "@bokehjs/core/build_views"
+import type * as p from "@bokehjs/core/properties"
 
 import {Button as BkButton, ButtonView as BkButtonView} from "@bokehjs/models/widgets/button"
 
-
 export class ButtonView extends BkButtonView {
   declare model: Button
 
@@ -12,15 +13,17 @@ export class ButtonView extends BkButtonView {
 
   override *children(): IterViews {
     yield* super.children()
-    if (this.tooltip != null)
+    if (this.tooltip != null) {
       yield this.tooltip
+    }
   }
 
   override async lazy_initialize(): Promise<void> {
     await super.lazy_initialize()
     const {tooltip} = this.model
-    if (tooltip != null)
+    if (tooltip != null) {
       this.tooltip = await build_view(tooltip, {parent: this})
+    }
   }
 
   override remove(): void {
@@ -63,7 +66,7 @@ export class Button extends BkButton {
   declare properties: Button.Props
   declare __view_type__: ButtonView
 
-  static __module__ = "panel.models.widgets"
+  static override __module__ = "panel.models.widgets"
 
   constructor(attrs?: Partial<Button.Attrs>) {
     super(attrs)
@@ -72,9 +75,9 @@ export class Button extends BkButton {
   static {
     this.prototype.default_view = ButtonView
 
-    this.define<Button.Props>(({Nullable, Ref, Number}) => ({
+    this.define<Button.Props>(({Nullable, Ref, Float}) => ({
       tooltip: [ Nullable(Ref(Tooltip)), null ],
-      tooltip_delay: [ Number, 500],
+      tooltip_delay: [ Float, 500],
     }))
   }
 }
