@@ -36,7 +36,7 @@ from .echarts import ECharts  # noqa
 from .equation import LaTeX  # noqa
 from .holoviews import HoloViews, Interactive  # noqa
 from .image import (  # noqa
-    GIF, ICO, JPG, PDF, PNG, SVG, Image,
+    GIF, ICO, JPG, PDF, PNG, SVG, Image, WebP,
 )
 from .ipywidget import IPyLeaflet, IPyWidget, Reacton  # noqa
 from .markup import (  # noqa
@@ -49,10 +49,15 @@ from .plot import (  # noqa
 )
 from .plotly import Plotly  # noqa
 from .streamz import Streamz  # noqa
+from .textual import Textual  # noqa
 from .vega import Vega  # noqa
 from .vizzu import Vizzu  # noqa
 from .vtk import VTK, VTKVolume  # noqa
 
+from ..param import (  # noqa, isort: skip
+    ParamFunction, ParamMethod, ParamRef, ReactiveExpr,
+)
+
 __all__ = (
     "Alert",
     "Audio",
@@ -73,16 +78,21 @@ __all__ = (
     "Markdown",
     "Matplotlib",
     "PaneBase",
+    "ParamFunction",
+    "ParamMethod",
+    "ParamRef",
     "panel",
     "PDF",
     "Perspective",
     "Plotly",
     "PNG",
+    "ReactiveExpr",
     "Reacton",
     "RGGPlot",
     "Str",
     "Streamz",
     "SVG",
+    "Textual",
     "Vega",
     "Video",
     "Vizzu",
