@@ -15,7 +15,6 @@ import param
 from bokeh.models import ColumnDataSource, ImportedStyleSheet
 from pyviz_comms import JupyterComm
 
-from ..io.resources import CDN_DIST
 from ..io.state import state
 from ..reactive import ReactiveData
 from ..util import datetime_types, lazy_load
@@ -29,10 +28,11 @@ if TYPE_CHECKING:
 
     from ..model.perspective import PerspectiveClickEvent
 
-DEFAULT_THEME = "material"
+DEFAULT_THEME = "pro"
 
 THEMES = [
-    'material', 'material-dark', 'monokai', 'solarized', 'solarized-dark', 'vaporwave'
+    'material', 'material-dark', 'monokai', 'solarized', 'solarized-dark',
+    'vaporwave', 'pro', 'pro-dark'
 ]
 
 class Plugin(Enum):
@@ -264,19 +264,23 @@ class Perspective(ModelPane, ReactiveData):
 
     :Example:
 
-    >>> Perspective(df, plugin='hypergrid', theme='material-dark')
+    >>> Perspective(df, plugin='hypergrid', theme='pro-dark')
     """
 
     aggregates = param.Dict(default=None, nested_refs=True, doc="""
       How to aggregate. For example {"x": "distinct count"}""")
 
     columns = param.List(default=None, nested_refs=True, doc="""
-        A list of source columns to show as columns. For example ["x", "y"]""")
+      A list of source columns to show as columns. For example ["x", "y"]""")
+
+    columns_config = param.Dict(default=None, nested_refs=True, doc="""
+      Column configuration allowing specification of formatters, coloring
+      and a variety of other attributes for each column.""")
 
     editable = param.Boolean(default=True, allow_None=True, doc="""
       Whether items are editable.""")
 
-    expressions = param.List(default=None, nested_refs=True, doc="""
+    expressions = param.ClassSelector(class_=(dict, list), default=None, nested_refs=True, doc="""
       A list of expressions computing new columns from existing columns.
       For example [""x"+"index""]""")
 
@@ -307,11 +311,17 @@ class Perspective(ModelPane, ReactiveData):
     plugin_config = param.Dict(default={}, nested_refs=True, doc="""
       Configuration for the PerspectiveViewerPlugin.""")
 
+    settings = param.Boolean(default=True, doc="""
+      Whether to show the settings menu.""")
+
     toggle_config = param.Boolean(default=True, doc="""
       Whether to show the config menu.""")
 
-    theme = param.ObjectSelector(default='material', objects=THEMES, doc="""
-      The style of the PerspectiveViewer. For example material-dark""")
+    theme = param.ObjectSelector(default='pro', objects=THEMES, doc="""
+      The style of the PerspectiveViewer. For example pro-dark""")
+
+    title = param.String(default=None, doc="""
+      Title for the Perspective viewer.""")
 
     priority: ClassVar[float | bool | None] = None
 
@@ -325,10 +335,6 @@ class Perspective(ModelPane, ReactiveData):
 
     _updates: ClassVar[bool] = True
 
-    _stylesheets: ClassVar[List[str]] = [
-        f'{CDN_DIST}css/perspective-datatable.css'
-    ]
-
     @classmethod
     def applies(cls, object):
         if isinstance(object, dict) and all(isinstance(v, (list, np.ndarray)) for v in object.values()):
@@ -373,6 +379,8 @@ class Perspective(ModelPane, ReactiveData):
 
     def _get_properties(self, doc, source=None):
         props = super()._get_properties(doc)
+        if 'theme' in props and 'material' in props['theme']:
+            props['theme'] = props['theme'].replace('material', 'pro')
         del props['object']
         if props.get('toggle_config'):
             props['height'] = self.height or 300
@@ -401,10 +409,10 @@ class Perspective(ModelPane, ReactiveData):
             else:
                 if len(array):
                     value = array[0]
-                    if isinstance(value, dt.date):
-                        schema[col] = 'date'
-                    elif isinstance(value, datetime_types):
+                    if isinstance(value, datetime_types) and type(value) is not dt.date:
                         schema[col] = 'datetime'
+                    elif isinstance(value, dt.date):
+                        schema[col] = 'date'
                     elif isinstance(value, str):
                         schema[col] = 'string'
                     elif isinstance(value, (float, np.floating)):
@@ -421,7 +429,7 @@ class Perspective(ModelPane, ReactiveData):
         from ..models.perspective import THEME_URL
         theme_url = f'{THEME_URL}{theme}.css'
         if self._bokeh_model is not None:
-            self._bokeh_model.__css_raw__ = self._bokeh_model.__css_raw__[:3] + [theme_url]
+            self._bokeh_model.__css_raw__ = self._bokeh_model.__css_raw__[:5] + [theme_url]
         return theme_url
 
     def _process_param_change(self, params):
@@ -431,6 +439,8 @@ class Perspective(ModelPane, ReactiveData):
             params['stylesheets'] = [
                 ImportedStyleSheet(url=ss) for ss in css
             ] + params.get('stylesheets', self.stylesheets)
+        if 'theme' in params and 'material' in params['theme']:
+            params['theme'] = params['theme'].replace('material', 'pro')
         props = super()._process_param_change(params)
         for p in ('columns', 'group_by', 'split_by'):
             if props.get(p):
@@ -441,6 +451,8 @@ class Perspective(ModelPane, ReactiveData):
             props['filters'] = [[str(col), *args] for col, *args in props['filters']]
         if props.get('aggregates'):
             props['aggregates'] = {str(col): agg for col, agg in props['aggregates'].items()}
+        if isinstance(props.get('expressions'), list):
+            props['expressions'] = {f'expression_{i}': exp for i, exp in enumerate(props['expressions'])}
         return props
 
     def _as_digit(self, col):
