@@ -24,12 +24,27 @@ from typing import (
 
 import param
 
+try:
+    from param import Skip
+except Exception:
+    class Skip(Exception):
+        """
+        Exception that allows skipping an update for function-level updates.
+        """
 from param.parameterized import (
-    classlist, discard_events, eval_function_with_deps, get_method_owner,
-    iscoroutinefunction,
+    Undefined, classlist, discard_events, eval_function_with_deps,
+    get_method_owner, iscoroutinefunction, resolve_ref, resolve_value,
 )
 from param.reactive import rx
 
+try:
+    from param import Skip
+except Exception:
+    class Skip(RuntimeError):
+        """
+        Exception that allows skipping an update for function-level updates.
+        """
+
 from .config import config
 from .io import state
 from .layout import (
@@ -761,14 +776,11 @@ class Param(PaneBase):
         return super().select(selector) + self.layout.select(selector)
 
 
-class ParamMethod(ReplacementPane):
+class ParamRef(ReplacementPane):
     """
-    ParamMethod panes wrap methods on parameterized classes and
-    rerenders the plot when any of the method's parameters change. By
-    default ParamMethod will watch all parameters on the class owning
-    the method or can be restricted to certain parameters by annotating
-    the method using the param.depends decorator. The method may
-    return any object which itself can be rendered as a Pane.
+    ParamRef wraps any valid parameter reference and resolves it
+    dynamically, re-rendering the output. If enabled it will attempt
+    to update the previously rendered component inplace.
     """
 
     defer_load = param.Boolean(default=None, doc="""
@@ -786,9 +798,13 @@ class ParamMethod(ReplacementPane):
         Whether to show a loading indicator while the pane is updating.
         Can be set as parameter or by setting panel.config.loading_indicator.""")
 
+    priority: ClassVar[float | bool | None] = 0
+
     def __init__(self, object=None, **params):
         if 'defer_load' not in params:
             params['defer_load'] = config.defer_load
+        if 'loading_indicator' not in params:
+            params['loading_indicator'] = ParamMethod.loading_indicator
         super().__init__(object, **params)
         self._async_task = None
         self._evaled = not (self.lazy or self.defer_load)
@@ -798,30 +814,20 @@ class ParamMethod(ReplacementPane):
             if not self.defer_load:
                 self._replace_pane()
 
-    @param.depends('object', watch=True)
+    @classmethod
+    def applies(cls, obj: Any) -> float | bool | None:
+        return bool(resolve_ref(obj))
+
     def _validate_object(self):
-        dependencies = getattr(self.object, '_dinfo', None)
-        if not dependencies or not dependencies.get('watch'):
-            return
-        fn_type = 'method' if type(self) is ParamMethod else 'function'
-        self.param.warning(
-            f"The {fn_type} supplied for Panel to display was declared "
-            f"with `watch=True`, which will cause the {fn_type} to be "
-            "called twice for any change in a dependent Parameter. "
-            "`watch` should be False when Panel is responsible for "
-            f"displaying the result of the {fn_type} call, while "
-            f"`watch=True` should be reserved for {fn_type}s that work "
-            "via side-effects, e.g. by modifying internal state of a "
-            "class or global state in an application's namespace."
-        )
+        return
 
     #----------------------------------------------------------------
     # Callback API
     #----------------------------------------------------------------
 
     @classmethod
-    def eval(self, function):
-        return eval_function_with_deps(function)
+    def eval(cls, ref):
+        return resolve_value(ref)
 
     async def _eval_async(self, awaitable):
         if self._async_task:
@@ -841,9 +847,20 @@ class ParamMethod(ReplacementPane):
                         self._inner_layout.append(new_obj)
                         self._pane = self._inner_layout[-1]
                     else:
-                        self._update_inner(new_obj)
+                        try:
+                            self._update_inner(new_obj)
+                        except Skip:
+                            pass
             else:
-                self._update_inner(await awaitable)
+                try:
+                    new = await awaitable
+                    if new is Skip or new is Undefined:
+                        raise Skip
+                    self._update_inner(new)
+                except Skip:
+                    self.param.log(
+                        param.DEBUG, 'Skip event was raised, skipping update.'
+                    )
         except Exception as e:
             if not curdoc or (has_context and curdoc.session_context):
                 raise e
@@ -862,7 +879,16 @@ class ParamMethod(ReplacementPane):
             if self.object is None:
                 new_object = Spacer()
             else:
-                new_object = self.eval(self.object)
+                try:
+                    new_object = self.eval(self.object)
+                    if new_object is Skip and new_object is Undefined:
+                        self._inner_layout.loading = False
+                        raise Skip
+                except Skip:
+                    self.param.log(
+                        param.DEBUG, 'Skip event was raised, skipping update.'
+                    )
+                    return
             if inspect.isawaitable(new_object) or isinstance(new_object, types.AsyncGeneratorType):
                 param.parameterized.async_executor(partial(self._eval_async, new_object))
                 return
@@ -884,7 +910,7 @@ class ParamMethod(ReplacementPane):
     def _update_pane(self, *events):
         callbacks = []
         for watcher in self._internal_callbacks:
-            obj = watcher.inst if watcher.inst is None else watcher.cls
+            obj = watcher.cls if watcher.inst is None else watcher.inst
             if obj is self:
                 callbacks.append(watcher)
                 continue
@@ -893,6 +919,86 @@ class ParamMethod(ReplacementPane):
         self._link_object_params()
         self._replace_pane()
 
+    def _get_model(
+        self, doc: Document, root: Optional[Model] = None,
+        parent: Optional[Model] = None, comm: Optional[Comm] = None
+    ) -> Model:
+        if not self._evaled:
+            deferred = self.defer_load and not state.loaded
+            if deferred:
+                state.onload(
+                    partial(self._replace_pane, force=True),
+                    threaded=bool(state._thread_pool)
+                )
+            self._replace_pane(force=not deferred)
+        return super()._get_model(doc, root, parent, comm)
+
+    def _link_object_params(self):
+        dep_params = resolve_ref(self.object)
+        if callable(self.object) and not (dep_params or self.lazy or self.defer_load or iscoroutinefunction(self.object)):
+            fn = getattr(self.object, '__bound_function__', self.object)
+            fn_name = getattr(fn, '__name__', repr(self.object))
+            self.param.warning(
+                f"The function {fn_name!r} does not have any dependencies "
+                "and will never update. Are you sure you did not intend "
+                "to depend on or bind a parameter or widget to this function? "
+                "If not simply call the function before passing it to Panel. "
+                "Otherwise, when passing a parameter as an argument, "
+                "ensure you pass at least one parameter and reference the "
+                "actual parameter object not the current value, i.e. use "
+                "object.param.parameter not object.parameter."
+            )
+        grouped = defaultdict(list)
+        for dep in dep_params:
+            grouped[id(dep.owner)].append(dep)
+        for group in grouped.values():
+            pobj = group[0].owner
+            watcher = pobj.param.watch(self._replace_pane, [dep.name for dep in group])
+            if isinstance(pobj, Reactive) and self.loading_indicator:
+                props = {dep.name: 'loading' for dep in group
+                         if dep.name in pobj._linkable_params}
+                if props:
+                    pobj.jslink(self._inner_layout, **props)
+            self._internal_callbacks.append(watcher)
+
+
+@param.depends(config.param.defer_load, watch=True)
+def _update_defer_load_default(default_value):
+    ParamRef.param.defer_load.default = default_value
+
+@param.depends(config.param.loading_indicator, watch=True)
+def _update_loading_indicator_default(default_value):
+    ParamRef.param.loading_indicator.default = default_value
+
+
+class ParamMethod(ParamRef):
+    """
+    ParamMethod panes wrap methods on parameterized classes and
+    rerenders the plot when any of the method's parameters change. By
+    default ParamMethod will watch all parameters on the class owning
+    the method or can be restricted to certain parameters by annotating
+    the method using the param.depends decorator. The method may
+    return any object which itself can be rendered as a Pane.
+    """
+
+    priority: ClassVar[float | bool | None] = 0.5
+
+    @param.depends('object', watch=True)
+    def _validate_object(self):
+        dependencies = getattr(self.object, '_dinfo', {})
+        if not dependencies or not dependencies.get('watch'):
+            return
+        self.param.warning(
+            "The method supplied for Panel to display was declared "
+            "with `watch=True`, which will cause the method to be "
+            "called twice for any change in a dependent Parameter. "
+            "`watch` should be False when Panel is responsible for "
+            "displaying the result of the method call, while "
+            "`watch=True` should be reserved for methods that work "
+            "via side-effects, e.g. by modifying internal state of a "
+            "class or global state in an application's namespace."
+        )
+
     def _link_object_params(self):
         parameterized = get_method_owner(self.object)
         params = parameterized.param.method_dependencies(self.object.__name__)
@@ -935,20 +1041,6 @@ class ParamMethod(ReplacementPane):
             watcher = pobj.param.watch(update_pane, ps, p.what)
             self._internal_callbacks.append(watcher)
 
-    def _get_model(
-        self, doc: Document, root: Optional[Model] = None,
-        parent: Optional[Model] = None, comm: Optional[Comm] = None
-    ) -> Model:
-        if not self._evaled:
-            deferred = self.defer_load and not state.loaded
-            if deferred:
-                state.onload(
-                    partial(self._replace_pane, force=True),
-                    threaded=bool(state._thread_pool)
-                )
-            self._replace_pane(force=not deferred)
-        return super()._get_model(doc, root, parent, comm)
-
     #----------------------------------------------------------------
     # Public API
     #----------------------------------------------------------------
@@ -957,15 +1049,12 @@ class ParamMethod(ReplacementPane):
     def applies(cls, obj: Any) -> float | bool | None:
         return inspect.ismethod(obj) and isinstance(get_method_owner(obj), param.Parameterized)
 
-@param.depends(config.param.defer_load, watch=True)
-def _update_defer_load_default(default_value):
-    ParamMethod.param.defer_load.default = default_value
+    @classmethod
+    def eval(cls, ref):
+        return eval_function_with_deps(ref)
 
-@param.depends(config.param.loading_indicator, watch=True)
-def _update_loading_indicator_default(default_value):
-    ParamMethod.param.loading_indicator.default = default_value
 
-class ParamFunction(ParamMethod):
+class ParamFunction(ParamRef):
     """
     ParamFunction panes wrap functions decorated with the param.depends
     decorator and rerenders the output when any of the function's
@@ -978,34 +1067,21 @@ class ParamFunction(ParamMethod):
 
     _applies_kw: ClassVar[bool] = True
 
-    def _link_object_params(self):
-        deps = getattr(self.object, '_dinfo', {})
-        dep_params = list(deps.get('dependencies', [])) + list(deps.get('kw', {}).values())
-        if not dep_params and not self.lazy and not self.defer_load and not iscoroutinefunction(self.object):
-            fn = getattr(self.object, '__bound_function__', self.object)
-            fn_name = getattr(fn, '__name__', repr(self.object))
-            self.param.warning(
-                f"The function {fn_name!r} does not have any dependencies "
-                "and will never update. Are you sure you did not intend "
-                "to depend on or bind a parameter or widget to this function? "
-                "If not simply call the function before passing it to Panel. "
-                "Otherwise, when passing a parameter as an argument, "
-                "ensure you pass at least one parameter and reference the "
-                "actual parameter object not the current value, i.e. use "
-                "object.param.parameter not object.parameter."
-            )
-        grouped = defaultdict(list)
-        for dep in dep_params:
-            grouped[id(dep.owner)].append(dep)
-        for group in grouped.values():
-            pobj = group[0].owner
-            watcher = pobj.param.watch(self._replace_pane, [dep.name for dep in group])
-            if isinstance(pobj, Reactive) and self.loading_indicator:
-                props = {dep.name: 'loading' for dep in group
-                         if dep.name in pobj._linkable_params}
-                if props:
-                    pobj.jslink(self._inner_layout, **props)
-            self._internal_callbacks.append(watcher)
+    @param.depends('object', watch=True)
+    def _validate_object(self):
+        dependencies = getattr(self.object, '_dinfo', {})
+        if not dependencies or not dependencies.get('watch'):
+            return
+        self.param.warning(
+            "The function supplied for Panel to display was declared "
+            "with `watch=True`, which will cause the function to be "
+            "called twice for any change in a dependent Parameter. "
+            "`watch` should be False when Panel is responsible for "
+            "displaying the result of the function call, while "
+            "`watch=True` should be reserved for functions that work "
+            "via side-effects, e.g. by modifying internal state of a "
+            "class or global state in an application's namespace."
+        )
 
     #----------------------------------------------------------------
     # Public API
@@ -1025,6 +1101,10 @@ class ParamFunction(ParamMethod):
             return None
         return False
 
+    @classmethod
+    def eval(self, ref):
+        return eval_function_with_deps(ref)
+
 
 class ReactiveExpr(PaneBase):
     """
@@ -1156,6 +1236,12 @@ class ReactiveExpr(PaneBase):
                     widgets.append(w)
         return self.widget_layout(*widgets)
 
+    def _get_model(
+        self, doc: Document, root: Optional['Model'] = None,
+        parent: Optional['Model'] = None, comm: Optional[Comm] = None
+    ) -> 'Model':
+        return self.layout._get_model(doc, root, parent, comm)
+
     def _generate_layout(self):
         panel = ParamFunction(self.object._callback)
         if not self.show_widgets:
