@@ -45,8 +45,7 @@ from .models.reactive_html import (
     DOMEvent, ReactiveHTML as _BkReactiveHTML, ReactiveHTMLParser,
 )
 from .util import (
-    BOKEH_JS_NAT, HTML_SANITIZER, classproperty, edit_readonly, escape,
-    updating,
+    HTML_SANITIZER, classproperty, edit_readonly, escape, updating,
 )
 from .viewable import Layoutable, Renderable, Viewable
 
@@ -111,6 +110,9 @@ class Syncable(Renderable):
     # panel/dist directory
     _stylesheets: ClassVar[List[str]] = []
 
+    # Property changes that should not trigger busy indicator
+    _busy__ignore = []
+
     __abstract = True
 
     def __init__(self, **params):
@@ -375,8 +377,9 @@ class Syncable(Renderable):
 
     def _process_events(self, events: Dict[str, Any]) -> None:
         self._log('received events %s', events)
-        with edit_readonly(state):
-            state._busy_counter += 1
+        if any(e for e in events if e not in self._busy__ignore):
+            with edit_readonly(state):
+                state._busy_counter += 1
         events = self._process_property_change(events)
         try:
             with edit_readonly(self):
@@ -404,8 +407,9 @@ class Syncable(Renderable):
             raise
         finally:
             self._log('finished processing events %s', events)
-            with edit_readonly(state):
-                state._busy_counter -= 1
+            if any(e for e in events if e not in self._busy__ignore):
+                with edit_readonly(state):
+                    state._busy_counter -= 1
 
     def _process_bokeh_event(self, doc: Document, event: Event) -> None:
         self._log('received bokeh event %s', event)
@@ -1218,8 +1222,8 @@ class ReactiveData(SyncableData):
         dtype = old_values.dtype
         converted: List | np.ndarray | None = None
         if dtype.kind == 'M':
-            if values.dtype.kind in 'if':
-                NATs = values == BOKEH_JS_NAT
+            if values.dtype.kind in 'if':# TODO: need to doublecheck if this is still needed
+                NATs = np.isnan(values)
                 converted = np.where(NATs, np.nan, values * 10e5).astype(dtype)
         elif dtype.kind == 'O':
             if (all(isinstance(ov, dt.date) for ov in old_values) and
@@ -1430,21 +1434,38 @@ class ReactiveHTMLMetaclass(ParameterizedMetaclass):
 
 class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
     """
-    ReactiveHTML provides bi-directional syncing of arbitrary HTML
-    attributes and DOM properties with parameters on the subclass.
+    The ReactiveHTML class enables you to create custom Panel components using HTML, CSS and/ or
+    Javascript and without the complexities of Javascript build tools.
+
+    A `ReactiveHTML` subclass provides bi-directional syncing of its parameters with arbitrary HTML
+    elements, attributes and properties. The key part of the subclass is the `_template`
+    variable. This is the HTML template that gets rendered and declares how to link parameters on the
+    subclass to HTML.
+
+    >>>    import panel as pn
+    >>>    import param
+    >>>    class CustomComponent(pn.reactive.ReactiveHTML):
+    ...
+    ...        index = param.Integer(default=0)
+    ...
+    ...        _template = '<img id="slideshow" src="https://picsum.photos/800/300?image=${index}" onclick="${_img_click}"></img>'
+    ...
+    ...        def _img_click(self, event):
+    ...            self.index += 1
+    >>>    CustomComponent(width=500, height=200).servable()
 
     HTML templates
     ~~~~~~~~~~~~~~
 
     A ReactiveHTML component is declared by providing an HTML template
     on the `_template` attribute on the class. Parameters are synced by
-    inserting them as template variables of the form `${parameter}`,
+    inserting them as template variables of the form `${parameter_name}`,
     e.g.:
 
-        <div class="${div_class}">${children}</div>
+        <div class="${parameter_1}">${parameter_2}</div>
 
-    will interpolate the div_class parameter on the class. In addition
-    to providing attributes we can also provide children to an HTML
+    will interpolate the parameter1 parameter on the class.
+    In addition to providing attributes we can also provide children to an HTML
     tag. By default any parameter referenced as a child will be
     treated as a Panel components to be rendered into the containing
     HTML. This makes it possible to use ReactiveHTML to lay out other
@@ -1476,17 +1497,17 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
     properties are synced when an event is fired. To make this possible
     the HTML element in question must be given a unique id, e.g.:
 
-        <input id="input"></input>
+        <input id="input_el"></input>
 
     Now we can use this name to declare set of `_dom_events` to
     subscribe to. The following will subscribe to change DOM events
     on the input element:
 
-       {'input': ['change']}
+       {'input_el': ['change']}
 
     Once subscribed the class may also define a method following the
     `_{node}_{event}` naming convention which will fire when the DOM
-    event triggers, e.g. we could define a `_input_change` method.
+    event triggers, e.g. we could define a `_input_el_change` method.
     Any such callback will be given a DOMEvent object as the first and
     only argument. The DOMEvent contains information about the event
     on the .data attribute and declares the type of event on the .type
@@ -1498,19 +1519,19 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
     Instead of declaring explicit DOM events Python callbacks can also
     be declared inline, e.g.:
 
-        <input id="input" onchange="${_input_change}"></input>
+        <input id="input_el" onchange="${_input_change}"></input>
 
     will look for an `_input_change` method on the ReactiveHTML
     component and call it when the event is fired.
 
     Additionally we can invoke pure JS scripts defined on the class, e.g.:
 
-        <input id="input" onchange="${script('some_script')}"></input>
+        <input id="input_el" onchange="${script('some_script')}"></input>
 
     This will invoke the following script if it is defined on the class:
 
         _scripts = {
-            'some_script': 'console.log(model, data, input, view)'
+            'some_script': 'console.log(model, data, input_el, view)'
        }
 
     Scripts
@@ -1521,13 +1542,13 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
     attribute changes. Let us say we have declared an input element
     with a synced value parameter:
 
-        <input id="input" value="${value}"></input>
+        <input id="input_el" value="${value}"></input>
 
     We can now declare a set of `_scripts`, which will fire whenever
     the value updates:
 
         _scripts = {
-            'value': 'console.log(model, data, input)'
+            'value': 'console.log(model, data, input_el)'
        }
 
     The Javascript is provided multiple objects in its namespace
@@ -1535,7 +1556,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
 
       * data :  The data model holds the current values of the synced
                 parameters, e.g. data.value will reflect the current
-                value of the input node.
+                value of the `input_el` node.
       * model:  The ReactiveHTML model which holds layout information
                 and information about the children and events.
       * state:  An empty state dictionary which scripts can use to
@@ -1545,7 +1566,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 `invalidate_layout` and `run_script` which allows
                 invoking other scripts.
       * <node>: All named DOM nodes in the HTML template, e.g. the
-                `input` node in the example above.
+                `input_el` node in the example above.
     """
 
     _child_config: ClassVar[Mapping[str, str]] = {}
