@@ -29,8 +29,9 @@ from ..io.model import add_to_doc
 from ..io.notebook import render_template
 from ..io.notifications import NotificationArea
 from ..io.resources import (
-    BUNDLE_DIR, CDN_DIST, ResourceComponent, _env, component_resource_path,
-    get_dist_path, loading_css, parse_template, resolve_custom_path,
+    BUNDLE_DIR, CDN_DIST, JS_VERSION, ResourceComponent, _env,
+    component_resource_path, get_dist_path, loading_css, parse_template,
+    resolve_custom_path, use_cdn,
 )
 from ..io.save import save
 from ..io.state import curdoc_locked, state
@@ -116,6 +117,8 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
         config_params = {
             p: v for p, v in params.items() if p in _base_config.param
         }
+        self._render_items: Dict[str, Tuple[Renderable, List[str]]]  = {}
+        self._render_variables: Dict[str, Any] = {}
         super().__init__(**{
             p: v for p, v in params.items() if p not in _base_config.param or p == 'name'
         })
@@ -130,8 +133,6 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
             self.nb_template = _env.from_string(nb_template)
         else:
             self.nb_template = nb_template or self.template
-        self._render_items: Dict[str, Tuple[Renderable, List[str]]]  = {}
-        self._render_variables: Dict[str, Any] = {}
         self._documents: List[Document] = []
         self._server = None
         self._layout = self._build_layout()
@@ -142,7 +143,9 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
         self._design = self.design(theme=self.theme)
 
     def _update_vars(self, *args) -> None:
-        self._render_variables['template_resources'] = self.resolve_resources()
+        """
+        Updates the render variables before the template is rendered.
+        """
 
     def _build_layout(self) -> Column:
         str_repr = Str(repr(self))
@@ -210,7 +213,8 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
 
         # Add all render items to the document
         objs, models = [], []
-        sizing_modes = {}
+        stylesheets, sizing_modes = {}, {}
+        tracked_models = set()
         for name, (obj, tags) in self._render_items.items():
 
             # Render root without pre-processing
@@ -227,6 +231,11 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
                 # pre-processor correctly operates on fake root
                 for sub in obj.select(Viewable):
                     submodel = sub._models.get(mref)
+                    for stylesheet in getattr(sub, '_stylesheets', []):
+                        if not stylesheet.endswith('.css'):
+                            continue
+                        sts_name = f'extra_{os.path.basename(stylesheet)}'
+                        stylesheets[sts_name] = stylesheet
                     if submodel is None:
                         continue
                     sub._models[ref] = submodel
@@ -241,7 +250,7 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
                 objs.append(obj)
                 models.append(model)
 
-            add_to_doc(model, document, hold=bool(comm))
+            tracked_models |= add_to_doc(model, document, hold=bool(comm), skip=tracked_models)
             document.on_session_destroyed(obj._server_destroy) # type: ignore
 
         # Here we ensure that the preprocessor is run across all roots
@@ -263,6 +272,8 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
             document.template = self.template
 
         self._update_vars()
+        resources = self.resolve_resources(extras={'css': stylesheets})
+        document._template_variables['template_resources'] = resources
         document._template_variables['sizing_modes'] = sizing_modes
         document._template_variables.update(self._render_variables)
         return document
@@ -317,7 +328,11 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
     # Public API
     #----------------------------------------------------------------
 
-    def resolve_resources(self, cdn: bool | Literal['auto'] = 'auto') -> ResourcesType:
+    def resolve_resources(
+        self,
+        cdn: bool | Literal['auto'] = 'auto',
+        extras: dict[str, dict[str, str]] | None = None
+    ) -> ResourcesType:
         """
         Resolves the resources required for this template component.
 
@@ -327,13 +342,16 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
             Whether to load resources from CDN or local server. If set
             to 'auto' value will be automatically determine based on
             global settings.
+        extras: dict[str, dict[str, str]] | None
+            Additional resources to add to the bundle. Valid resource
+            types include js, js_modules and css.
 
         Returns
         -------
         Dictionary containing JS and CSS resources.
         """
         cls = type(self)
-        resource_types = super().resolve_resources(cdn=cdn)
+        resource_types = super().resolve_resources(cdn=cdn, extras=extras)
         js_files = resource_types['js']
         js_modules = resource_types['js_modules']
         css_files = resource_types['css']
@@ -341,8 +359,11 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
 
         clsname = cls.__name__
         name = clsname.lower()
+        cdn = use_cdn() if cdn == 'auto' else cdn
         dist_path = get_dist_path(cdn=cdn)
+        version_suffix = f'?v={JS_VERSION}'
 
+        css_files['loading'] = f'{dist_path}css/loading.css{version_suffix}'
         raw_css.extend(list(self.config.raw_css) + [loading_css(
             config.loading_spinner, config.loading_color, config.loading_max_height
         )])
@@ -384,7 +405,7 @@ class BaseTemplate(param.Parameterized, MimeRenderMixin, ServableMixin, Resource
 
             css_file = os.path.basename(css)
             if (BUNDLE_DIR / tmpl_name / css_file).is_file():
-                css_files[f'base_{css_file}'] = dist_path + f'bundled/{tmpl_name}/{css_file}'
+                css_files[f'base_{css_file}'] = f'{dist_path}bundled/{tmpl_name}/{css_file}{version_suffix}'
             elif isurl(css):
                 css_files[f'base_{css_file}'] = css
             elif resolve_custom_path(self, css):
@@ -552,7 +573,10 @@ class TemplateActions(ReactiveHTML):
     _template: ClassVar[str] = ""
 
     _scripts: ClassVar[Dict[str, List[str] | str]] = {
-        'open_modal': ["document.getElementById('pn-Modal').style.display = 'block'"],
+        'open_modal': ["""
+          document.getElementById('pn-Modal').style.display = 'block'
+          window.dispatchEvent(new Event('resize'));
+        """],
         'close_modal': ["document.getElementById('pn-Modal').style.display = 'none'"],
     }
 
@@ -671,7 +695,7 @@ class BasicTemplate(BaseTemplate):
         tmpl_string = self._template.read_text(encoding='utf-8')
         try:
             template = _env.get_template(str(self._template.relative_to(Path(__file__).parent)))
-        except jinja2.exceptions.TemplateNotFound:
+        except (jinja2.exceptions.TemplateNotFound, ValueError):
             template = parse_template(tmpl_string)
 
         if 'header' not in params:
@@ -702,14 +726,15 @@ class BasicTemplate(BaseTemplate):
 
         super().__init__(template=template, **params)
         self._js_area = HTML(margin=0, width=0, height=0)
-        if 'embed(roots.js_area)' in tmpl_string:
+        state_roots = '{% block state_roots %}' in tmpl_string
+        if state_roots or 'embed(roots.js_area)' in tmpl_string:
             self._render_items['js_area'] = (self._js_area, [])
-        if 'embed(roots.actions)' in tmpl_string:
+        if state_roots or 'embed(roots.actions)' in tmpl_string:
             self._render_items['actions'] = (self._actions, [])
-        if 'embed(roots.notifications)' in tmpl_string and self.notifications:
+        if (state_roots or 'embed(roots.notifications)' in tmpl_string) and self.notifications:
             self._render_items['notifications'] = (self.notifications, [])
             self._render_variables['notifications'] = True
-        if config.browser_info and 'embed(roots.browser_info)' in tmpl_string and state.browser_info:
+        if config.browser_info and ('embed(roots.browser_info)' in tmpl_string or state_roots) and state.browser_info:
             self._render_items['browser_info'] = (state.browser_info, [])
             self._render_variables['browser_info'] = True
         self._update_busy()
