@@ -1,13 +1,14 @@
+import asyncio
 import os
-
-import pytest
+import pathlib
+import tempfile
 
 from panel.io.location import Location
 from panel.io.reload import (
-    _check_file, _modules, _reload_on_update, _watched_files, in_denylist,
-    record_modules, watch,
+    _check_file, _modules, _watched_files, in_denylist, record_modules, watch,
 )
 from panel.io.state import state
+from panel.tests.util import async_wait_until
 
 
 def test_record_modules_not_stdlib():
@@ -18,7 +19,7 @@ def test_record_modules_not_stdlib():
 
 def test_check_file():
     modify_times = {}
-    _check_file(modify_times, __file__)
+    _check_file(__file__, modify_times)
     assert modify_times[__file__] == os.stat(__file__).st_mtime
 
 def test_file_in_denylist():
@@ -36,16 +37,21 @@ def test_watch():
     # Cleanup
     _watched_files.clear()
 
-@pytest.mark.flaky(reruns=3)
-def test_reload_on_update():
+async def test_reload_on_update(server_document, watch_files):
     location = Location()
-    state._location = location
-    filepath = os.path.abspath(__file__)
-    watch(filepath)
-    modify_times = {filepath: os.stat(__file__).st_mtime-1}
-    _reload_on_update(modify_times)
-    assert location.reload
+    state._locations[server_document] = location
+    state._loaded[server_document] = True
+    with tempfile.NamedTemporaryFile() as temp:
+        # Write to file and wait for filesystem to perform write
+        temp.write(b'Foo')
+        temp.flush()
+        await asyncio.sleep(0.1)
 
-    # Cleanup
-    _watched_files.clear()
-    state._location = None
+        watch_files(temp.name)
+        await asyncio.sleep(0.1)
+
+        temp.write(b'Bar')
+        temp.flush()
+        pathlib.Path(temp.name).touch()
+
+        await async_wait_until(lambda: location.reload)
