@@ -5,7 +5,7 @@ from bokeh.models import Column as BkColumn, Div, Row as BkRow
 
 from panel.chat import ChatInterface
 from panel.layout import (
-    Accordion, Card, Column, Row, Spacer, Tabs, WidgetBox,
+    Accordion, Card, Column, FlexBox, Row, Spacer, Tabs, WidgetBox,
 )
 from panel.layout.base import ListPanel, NamedListPanel
 from panel.pane import Bokeh, Markdown
@@ -569,7 +569,6 @@ def test_no_expand_fixed(panel, document, comm):
 
     assert model.sizing_mode == 'fixed'
 
-
 @pytest.mark.parametrize('scroll_param', ["auto_scroll_limit", "scroll", "scroll_button_threshold", "view_latest"])
 def test_column_scroll_params_sets_scroll(scroll_param, document, comm):
     if scroll_param not in ["auto_scroll_limit", "scroll_button_threshold"]:
@@ -580,9 +579,10 @@ def test_column_scroll_params_sets_scroll(scroll_param, document, comm):
     assert getattr(col, scroll_param)
     assert col.scroll
 
-def test_pass_objects_ref(document, comm):
+@pytest.mark.parametrize('layout', [Row, Column, FlexBox])
+def test_pass_objects_ref(document, comm, layout):
     multi_select = MultiSelect(options=['foo', 'bar', 'baz'], value=['bar', 'baz'])
-    col = Column(objects=multi_select)
+    col = layout(objects=multi_select)
     col.get_root(document, comm=comm)
 
     assert len(col.objects) == 2
