@@ -8,6 +8,8 @@ except Exception:
 
 pydeck_available = pytest.mark.skipif(pydeck is None, reason="requires pydeck")
 
+from bokeh.core.serialization import Serializer
+
 from panel.models.deckgl import DeckGLPlot
 from panel.pane import DeckGL, PaneBase, panel
 
@@ -159,3 +161,56 @@ def test_deckgl_insert_layer(document, comm):
     assert cds1.data['b'] is b_vals
     assert np.array_equal(cds2.data['b'], np.array([3, 9]))
     assert np.array_equal(cds2.data['c'], np.array([1, 3]))
+
+@pydeck_available
+def test_pydeck_mapbox_api_key_issue_5790(document, comm):
+    deck_wo_key = pydeck.Deck()
+    pane_w_key = DeckGL(deck_wo_key, mapbox_api_key="ABC")
+
+    model = pane_w_key.get_root(document, comm=comm)
+    assert model.mapbox_api_key == "ABC"
+
+@pydeck_available
+def test_pydeck_no_min_max_zoom_issue_5790(document, comm):
+    state_w_no_min_max_zoom = {
+        "latitude": 37.7749,
+        "longitude": -122.4194,
+        "zoom": 10,
+        "bearing": 0,
+        "pitch": 0,
+    }
+    view_state = pydeck.ViewState(**state_w_no_min_max_zoom)
+    deck = pydeck.Deck(initial_view_state=view_state)
+    pane = DeckGL(deck)
+
+    model = pane.get_root(document, comm=comm)
+    assert model.initialViewState == state_w_no_min_max_zoom
+
+@pydeck_available
+def test_pydeck_type_string_can_be_serialized_issue_5790(document, comm):
+    serializer = Serializer(references=document.models.synced_references)
+    data = [
+                {
+                    "name": "24th St. Mission (24TH)",
+                    "code": "24",
+                    "address": "2800 Mission Street, San Francisco CA 94110",
+                    "entries": 12817,
+                    "exits": 12529,
+                    # "coordinates": [-122.418466, 37.752254]
+                }
+    ]
+
+
+    layer = pydeck.Layer(
+        "TextLayer",
+        data,
+        get_text_anchor=pydeck.types.String("middle"),
+        get_alignment_baseline=pydeck.types.String("center"),
+        size_units = pydeck.types.String("meters")   # <--- The key addition to switch to meters as the units.
+    )
+    deck = pydeck.Deck(layers=[layer])
+    pane = DeckGL(deck)
+
+    model = pane.get_root(document, comm=comm)
+    serializer.serialize(model)
+    assert Serializer._encoders.pop(pydeck.types.String)
