@@ -18,9 +18,9 @@ except Exception:
 plotly_available = pytest.mark.skipif(hv_plotly is None, reason="requires plotly backend")
 
 from bokeh.models import (
-    Column as BkColumn, ColumnDataSource, GlyphRenderer, GridPlot,
-    ImportedStyleSheet, Line, Row as BkRow, Scatter, Select as BkSelect,
-    Slider as BkSlider, Spacer as BkSpacer,
+    Column as BkColumn, ColumnDataSource, GlyphRenderer, GridPlot, Line,
+    Row as BkRow, Scatter, Select as BkSelect, Slider as BkSlider,
+    Spacer as BkSpacer,
 )
 from bokeh.plotting import figure
 
@@ -30,9 +30,7 @@ from panel.depends import bind
 from panel.layout import (
     Column, FlexBox, HSpacer, Row,
 )
-from panel.pane import (
-    HoloViews, PaneBase, Plotly, panel,
-)
+from panel.pane import HoloViews, PaneBase, panel
 from panel.tests.util import hv_available, mpl_available
 from panel.theme import Native
 from panel.util.warnings import PanelDeprecationWarning
@@ -211,6 +209,7 @@ def test_holoviews_pane_reflect_responsive_bind_function(document, comm):
 
     assert col.sizing_mode == 'fixed'
 
+@pytest.mark.usefixtures("hv_plotly")
 @hv_available
 @plotly_available
 def test_holoviews_pane_reflect_responsive_plotly(document, comm):
@@ -229,13 +228,13 @@ def test_holoviews_pane_reflect_responsive_plotly(document, comm):
     assert pane.sizing_mode is None
 
 
+@pytest.mark.usefixtures("hv_plotly")
 @hv_available
 @plotly_available
 def test_holoviews_pane_inherits_design_stylesheets(document, comm):
     curve = hv.Curve([1, 2, 3]).opts(responsive=True, backend='plotly')
     pane = HoloViews(curve, backend='plotly')
 
-
     # Create pane
     row = pane.get_root(document, comm=comm)
 
@@ -243,9 +242,7 @@ def test_holoviews_pane_inherits_design_stylesheets(document, comm):
 
     plotly_model = row.children[0]
 
-    assert len(plotly_model.stylesheets) == 6
-    stylesheets = [st.url for st in plotly_model.stylesheets if isinstance(st, ImportedStyleSheet)]
-    assert all(st in stylesheets for st in Plotly._stylesheets)
+    assert len(plotly_model.stylesheets) == 5
 
 @hv_available
 def test_holoviews_widgets_from_dynamicmap(document, comm):
@@ -530,6 +527,7 @@ def test_holoviews_widgets_explicit_widget_instance_override():
     assert widgets[0] is widget
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_linked_axes(document, comm):
     c1 = hv.Curve([1, 2, 3])
@@ -545,6 +543,7 @@ def test_holoviews_linked_axes(document, comm):
     assert p1.y_range is p2.y_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_linked_axes_flexbox(document, comm):
     c1 = hv.Curve([1, 2, 3])
@@ -560,6 +559,7 @@ def test_holoviews_linked_axes_flexbox(document, comm):
     assert p1.y_range is p2.y_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_linked_axes_merged_ranges(document, comm):
     c1 = hv.Curve([1, 2, 3])
@@ -577,6 +577,7 @@ def test_holoviews_linked_axes_merged_ranges(document, comm):
     assert p1.y_range.end == 4.4
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_linked_x_axis(document, comm):
     c1 = hv.Curve([1, 2, 3])
@@ -592,6 +593,7 @@ def test_holoviews_linked_x_axis(document, comm):
     assert p1.y_range is not p2.y_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_axiswise_not_linked_axes(document, comm):
     c1 = hv.Curve([1, 2, 3])
@@ -607,6 +609,7 @@ def test_holoviews_axiswise_not_linked_axes(document, comm):
     assert p1.y_range is not p2.y_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_shared_axes_opt_not_linked_axes(document, comm):
     c1 = hv.Curve([1, 2, 3])
@@ -622,6 +625,7 @@ def test_holoviews_shared_axes_opt_not_linked_axes(document, comm):
     assert p1.y_range is not p2.y_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_not_linked_axes(document, comm):
     c1 = hv.Curve([1, 2, 3])
@@ -640,6 +644,7 @@ def test_holoviews_not_linked_axes(document, comm):
     assert p1.y_range is not p2.y_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_link_across_panes(document, comm):
     from bokeh.models.tools import RangeTool
@@ -664,6 +669,7 @@ def test_holoviews_link_across_panes(document, comm):
     assert range_tool.x_range == p2.x_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_link_after_adding_item(document, comm):
     from bokeh.models.tools import RangeTool
@@ -692,6 +698,7 @@ def test_holoviews_link_after_adding_item(document, comm):
     assert range_tool.x_range == p2.x_range
 
 
+@pytest.mark.usefixtures("hv_bokeh")
 @hv_available
 def test_holoviews_link_within_pane(document, comm):
     from bokeh.models.tools import RangeTool
@@ -725,11 +732,9 @@ def test_holoviews_property_override_old_method(document, comm):
 
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", PanelDeprecationWarning)
-        pane = pn.panel(c1, backend='bokeh', background='red',
-                        css_classes=['test_class'])
+        pane = pn.panel(c1, backend='bokeh', css_classes=['test_class'])
     model = pane.get_root(document, comm=comm)
 
-    assert model.styles["background"] == 'red'
     assert model.children[0].css_classes == ['test_class']
 
 @hv_available
