@@ -89,16 +89,12 @@ class IconMixin(Widget):
         return super()._process_param_change(params)
 
 
-class _ClickButton(_ButtonBase, IconMixin):
+class _ClickButton(Widget):
 
     __abstract = True
 
     _event: ClassVar[str] = 'button_click'
 
-    _source_transforms: ClassVar[Mapping[str, str | None]] = {
-        'button_style': None,
-    }
-
     def _get_model(
         self, doc: Document, root: Optional[Model] = None,
         parent: Optional[Model] = None, comm: Optional[Comm] = None
@@ -156,7 +152,7 @@ class _ClickButton(_ButtonBase, IconMixin):
         return Callback(self, code=callbacks, args=args)
 
 
-class Button(_ClickButton, TooltipMixin):
+class Button(_ButtonBase, _ClickButton, IconMixin, TooltipMixin):
     """
     The `Button` widget allows triggering events when the button is
     clicked.
@@ -242,7 +238,7 @@ class Button(_ClickButton, TooltipMixin):
         links = {'event:'+self._event if p == 'value' else p: v for p, v in links.items()}
         return super().jslink(target, code, args, bidirectional, **links)
 
-    def _process_event(self, event: param.parameterized.Event) -> None:
+    def _process_event(self, event: ButtonClick) -> None:
         self.param.trigger('value')
         self.clicks += 1
 
@@ -304,7 +300,7 @@ class Toggle(_ButtonBase, IconMixin):
                 lambda x: x.active, 'active', 'cb_obj.active')
 
 
-class MenuButton(_ClickButton):
+class MenuButton(_ButtonBase, _ClickButton, IconMixin):
     """
     The `MenuButton` widget allows specifying a list of menu items to
     select from triggering events when the button is clicked.
@@ -352,7 +348,7 @@ class MenuButton(_ClickButton):
         self._register_events('button_click', model=model, doc=doc, comm=comm)
         return model
 
-    def _process_event(self, event):
+    def _process_event(self, event: ButtonClick):
         if isinstance(event, MenuItemClick):
             item = event.item
         elif isinstance(event, ButtonClick):
